@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2003-2007 Tim Kientzle
+ * Copyright (c) 2003-2009 Tim Kientzle
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,8 +26,13 @@
 __FBSDID("$FreeBSD$");
 
 #include "../cpio.h"
+#include "err.h"
 
-#if defined(__CYGWIN__)
+#if !defined(_WIN32)
+#define ROOT "root"
+static int root_uids[] = { 0 };
+static int root_gids[] = { 0 };
+#elif defined(__CYGWIN__)
 /* On cygwin, the Administrator user most likely exists (unless
  * it has been renamed or is in a non-English localization), but
  * its primary group membership depends on how the user set up
@@ -37,57 +42,70 @@ __FBSDID("$FreeBSD$");
  *       Use CreateWellKnownSID() and LookupAccountName()?
  */
 #define ROOT "Administrator"
-#define ROOT_UID 500
-#define ROOT_GID1 513
-#define ROOT_GID2 545
-#define ROOT_GID3 544
-#else
-#define ROOT "root"
-#define ROOT_UID 0
-#define ROOT_GID 0
+static int root_uids[] = { 500 };
+static int root_gids[] = { 513, 545, 544 };
 #endif
 
+#if defined(ROOT)
+static int
+int_in_list(int i, int *l, size_t n)
+{
+	while (n-- > 0)
+		if (*l++ == i)
+			return (1);
+	failure("%d", i);
+	return (0);
+}
+#endif
 
 DEFINE_TEST(test_owner_parse)
 {
-#if defined(_WIN32) && !defined(__CYGWIN__)
-	/* TODO: Does this need cygwin style handling of uid/gid ? */
-	skipping("Windows cannot handle uid/gid as UNIX like system");
+#if !defined(ROOT)
+	skipping("No uid/gid configuration for this OS");
 #else
 	int uid, gid;
 
-	cpio_progname = "Ignore this message";
-
-	assertEqualInt(0, owner_parse(ROOT, &uid, &gid));
-	assertEqualInt(ROOT_UID, uid);
+	assert(NULL == owner_parse(ROOT, &uid, &gid));
+	assert(int_in_list(uid, root_uids,
+		sizeof(root_uids)/sizeof(root_uids[0])));
 	assertEqualInt(-1, gid);
 
 
-	assertEqualInt(0, owner_parse(ROOT ":", &uid, &gid));
-	assertEqualInt(ROOT_UID, uid);
-#if defined(__CYGWIN__)
-	{
-		int gidIsOneOf = (ROOT_GID1 == gid)
-			|| (ROOT_GID2 == gid)
-			|| (ROOT_GID3 == gid);
-		assertEqualInt(1, gidIsOneOf);
-	}
-#else
-	assertEqualInt(ROOT_GID, gid);
-#endif
+	assert(NULL == owner_parse(ROOT ":", &uid, &gid));
+	assert(int_in_list(uid, root_uids,
+		sizeof(root_uids)/sizeof(root_uids[0])));
+	assert(int_in_list(gid, root_gids,
+		sizeof(root_gids)/sizeof(root_gids[0])));
 
-	assertEqualInt(0, owner_parse(ROOT ".", &uid, &gid));
-	assertEqualInt(ROOT_UID, uid);
-#if defined(__CYGWIN__)
-	{
-		int gidIsOneOf = (ROOT_GID1 == gid)
-			|| (ROOT_GID2 == gid)
-			|| (ROOT_GID3 == gid);
-		assertEqualInt(1, gidIsOneOf);
-	}
-#else
-	assertEqualInt(ROOT_GID, gid);
-#endif
+	assert(NULL == owner_parse(ROOT ".", &uid, &gid));
+	assert(int_in_list(uid, root_uids,
+		sizeof(root_uids)/sizeof(root_uids[0])));
+	assert(int_in_list(gid, root_gids,
+		sizeof(root_gids)/sizeof(root_gids[0])));
+
+	assert(NULL == owner_parse("111", &uid, &gid));
+	assertEqualInt(111, uid);
+	assertEqualInt(-1, gid);
+
+	assert(NULL == owner_parse("112:", &uid, &gid));
+	assertEqualInt(112, uid);
+	/* Can't assert gid, since we don't know gid for user #112. */
+
+	assert(NULL == owner_parse("113.", &uid, &gid));
+	assertEqualInt(113, uid);
+	/* Can't assert gid, since we don't know gid for user #113. */
+
+	assert(NULL == owner_parse(":114", &uid, &gid));
+	assertEqualInt(-1, uid);
+	assertEqualInt(114, gid);
+
+	assert(NULL == owner_parse(".115", &uid, &gid));
+	assertEqualInt(-1, uid);
+	assertEqualInt(115, gid);
+
+	assert(NULL == owner_parse("116:117", &uid, &gid));
+	assertEqualInt(116, uid);
+	assertEqualInt(117, gid);
 
 	/*
 	 * TODO: Lookup current user/group name, build strings and
@@ -95,18 +113,9 @@ DEFINE_TEST(test_owner_parse)
 	 * users.
 	 */
 
-	/*
-	 * TODO: Rework owner_parse to either return a char * pointing
-	 * to an error message or accept a function pointer to an
-	 * error-reporting routine so that the following tests don't
-	 * generate any output.
-	 *
-	 * Alternatively, redirect stderr temporarily to suppress the output.
-	 */
-
-	assertEqualInt(1, owner_parse(":nonexistentgroup", &uid, &gid));
-	assertEqualInt(1, owner_parse(ROOT ":nonexistentgroup", &uid, &gid));
-	assertEqualInt(1,
+	assert(NULL != owner_parse(":nonexistentgroup", &uid, &gid));
+	assert(NULL != owner_parse(ROOT ":nonexistentgroup", &uid, &gid));
+	assert(NULL !=
 	    owner_parse("nonexistentuser:nonexistentgroup", &uid, &gid));
 #endif
 }
